"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readLog = exports.log = exports.defaultLevels = void 0;
const general_formatter_1 = require("general-formatter");
const readline_1 = __importDefault(require("readline"));
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
/**
 * @description Object with list of default levels
 */
exports.defaultLevels = {
    info: {
        value: 'INFO',
        color: '34',
        writeToFile: true
    },
    database: {
        value: 'DATABASE',
        color: '32',
        writeToFile: true
    },
    debug: {
        value: 'DEBUG',
        color: '36',
        writeToFile: false
    },
    access: {
        value: 'ACCESS',
        color: '35',
        writeToFile: true
    },
    error: {
        value: 'ERROR',
        color: '31',
        writeToFile: true
    },
    fatal: {
        value: 'FATAL',
        color: '31;1',
        writeToFile: true
    }
};
const configData = {
    logDir: './logs',
    dateFormat: 'en-US',
    levels: exports.defaultLevels
};
/**
 * @summary Main logging function
 * @description Logs and write to the console
 * @param {LogOptions} options
 * @note Available levels: info, database, debug, access, error, and fatal
 * @note If the level name does not exist, the default level info will be added
 * @return {void} void
 */
function log({ levelName, message, error }) {
    const levels = configData.levels;
    const targetLevel = levels.hasOwnProperty(levelName)
        ? levels[levelName]
        : levels.info;
    if (error !== undefined) {
        message = `${error.name} ${error.message} ${error.stack}`;
    }
    // Write to the console
    writeToConsole(targetLevel, message);
    if (targetLevel.writeToFile) {
        writeToFile(targetLevel, message, error);
    }
}
exports.log = log;
/**
 * @description Gets the date and formatted time in MM/DD/YYYYTHH:MM:SS:ms
 * @returns {string} date
 */
function getCurrentDate() {
    const date = new Date();
    const formatted = (0, general_formatter_1.convertToDateString)({
        date: date.toString(),
        countryCode: 'en-US'
    });
    const completeDate = `${formatted}T${date.getHours()}:${date.getMinutes()}:${date.getSeconds()}`;
    return completeDate;
}
/**
 * @description Write to the console
 * @param {Level} targetLevel
 * @param {string} message
 */
function writeToConsole(targetLevel, message) {
    const header = `\x1b[${targetLevel.color}m[${targetLevel.value}] - ${getCurrentDate()}\x1b[0m`;
    console.log(`${header}: ${message}`);
}
/**
 * @description Write to file
 * @param {Level} targetLevel
 * @param {string} message
 * @param {Error | undefined} error
 */
function writeToFile(targetLevel, message, error) {
    const logDir = configData.logDir;
    let data = '';
    if (error !== undefined) {
        data = `{"level": "${targetLevel.value}", "errorName": "${error.name}", "errorMessage": "${error.message}", "timeStamp": "${getCurrentDate()}"}\r\n`;
    }
    else {
        data = `{"level": "${targetLevel.value}", "message": "${message}", "timeStamp": "${getCurrentDate()}"}\r\n`;
    }
    if (!(0, fs_1.existsSync)(logDir)) {
        log({
            levelName: 'debug',
            message: './logs directory does not exist. Creating one in root directory ...'
        });
        (0, fs_1.mkdirSync)(logDir);
    }
    const options = {
        encoding: 'utf8',
        mode: 438
    };
    try {
        (0, fs_1.appendFileSync)(`${logDir}/${targetLevel.value.toLowerCase()}.log`, data, options);
    }
    catch (error) {
        log({ levelName: 'debug', message: '', error });
    }
}
/**
 * @description Gets the logs of specified file
 * @param {string} fileName
 * @returns {Promise<JSON[]>} logs
 */
function readLog(fileName) {
    return __awaiter(this, void 0, void 0, function* () {
        const logDir = configData.logDir;
        return yield new Promise((resolve, reject) => {
            const file = path_1.default.join(logDir, (fileName === null || fileName === void 0 ? void 0 : fileName.includes('.')) ? fileName : `${fileName}.log`);
            const lineReader = readline_1.default.createInterface({
                input: (0, fs_1.createReadStream)(file)
            });
            const logs = [];
            lineReader.on('line', (line) => logs.push(JSON.parse(line)));
            lineReader.on('close', () => {
                log({
                    levelName: 'access',
                    message: `${fileName.toUpperCase()} logs have been accessed`
                });
                console.table(logs);
                resolve(logs);
            });
            lineReader.on('error', (error) => reject(error));
        });
    });
}
exports.readLog = readLog;
